<?php

namespace Quivers\Components;

require_once("Library/stripe-php/init.php"); // Stripe SDK Import
use Quivers\Components\SettingService;
use Quivers\Components\Logtracking;

class QuiversStripe
{

    private $session;
    private $settingsService;

    public function __construct(SettingService $settingsService, Logtracking $logTracking)
    {
        $this->session = Shopware()->Session();
        $this->settingsService = $settingsService;
        $this->logTracking = $logTracking;
    }

    /**
     * Create Stripe Customer.
    */
    public function createPaymentResponse($user, $data)
    {
        $startTime = microtime(true);
        $payment_token = $this->session["payment_token"];

        $secret_key = $this->settingsService->get('secret_key', 'stripe_settings');

        $this->minAmount = $this->settingsService->get('min_order_total', 'stripe_settings');
        
        $orderVariables = $this->session['sOrderVariables'];
        
        \Stripe\Stripe::setApiKey($secret_key);
        
        $customerEmail = $user['additional']['user']['email'];


        try {
            // check if user with same email already exists in Stripe account.
            $customer = false;
            $customersList = \Stripe\Customer::all(array(
                "limit" => 1,
                "email" => $customerEmail,
            ));

            if(count($customersList["data"])>0) {
                $customer = $customersList["data"][0];
            }
               
            // create a new stripe customer if it doesn't exists.
            if(!$customer){
                $customer = \Stripe\Customer::create(array(
                'source' => $payment_token,
                'email' => $customerEmail,
                ));

            }

            $customerId = $customer["id"];

            // add source to customer
             try {
                    $customer->sources->create(array("source" => $payment_token));
                }
            catch (Exception $e) {
            }

        }
        catch (\Exception $e) {
        }

        $paymentIntent = $this->createPaymentIntent($customer, $data);

        //Log Tracking
        $order_data = ["Customer Id"=>$customerId, "Email"=>$customerEmail];
        $response = ["Response"=>$paymentIntent["result"], "Amount"=>$data["amount"]];
        $this->logTracking->payment_request($order_data, "stripe", date('r', $startTime), $response, $paymentIntent["result"]["status"]);
        return $paymentIntent;

    }

    /**
     * Create Payment Intent. Check if 3D secure Authentication is required.
    */
    private function createPaymentIntent($customer, $data) 
    {
        $payment_token = $this->session["payment_token"];
        $amount = $data["amount"];
        $currency = $data["currency"];
        $stripeCustomerId= $customer['id'];
       
        $chargeAmount = $this->formatCurrencyAmount($amount, $currency);
        $orderVariables = $this->session['sOrderVariables'];
        $email = $orderVariables["sUserData"]["additional"]["user"]["email"];

        $redirectData = [
                'controller' => 'checkout',
                'action' => 'shippingPayment',
                'error_code' => 1,
        ];
        $selectedCard =  $this->session['QuiversStripeSelectedCard'];

        $statement_descriptor = $orderVariables["payment_descriptor"];
        if(strlen($statement_descriptor) > 10){
            $statement_descriptor = substr($statement_descriptor, 0, 10);
        }
        try {
            // $request = array(
            //     'customer' => $customer->id,
            //     'source' => $payment_token,
            //     'capture' => false,
            //     'amount' => $chargeAmount,
            //     'currency' => $currency,
            //     'description' => sprintf('%s', $email),
            //     'statement_descriptor_suffix' => $statement_descriptor != null ? $statement_descriptor : 'QuiversInc'
            // );
            $request = array(
                'customer' => $customer->id,
                'source' =>$payment_token,
                "capture_method" => "manual",
                'confirm' => false,
                'confirmation_method' => 'automatic',
                'amount' => $chargeAmount,
                'currency' => $currency,
                'description' => sprintf('%s', $email),
                'statement_descriptor_suffix' => $statement_descriptor != null ? $statement_descriptor : 'QuiversInc'
            );
            if($orderVariables['destination'] === true){
                $merchantId = $orderVariables['merchantId'];
                if($merchantId != '0'){
                    $transferData = array(
                        "destination" => $merchantId,
                        "amount" => $chargeAmount
                    );
                    $onBehalfOf = $merchantId;
                    $request['transfer_data'] = $transferData;
                    $request['on_behalf_of'] = $onBehalfOf;
                }
            }

            // Captcha Verification
            $captchaResponse = $this->session["stripe_captcha_response"];  
            if ($captchaResponse["success"]){
                // $charge = \stripe\charge::create($request);
                $intent = \Stripe\PaymentIntent::create($request);
                $intent->confirm([
                    'return_url' => $this->assembleShopwareUrl(['controller' => 'StripePayment','action' => 'completeRedirectFlow'])
                ]);
                // return ['error_code' => 0,
                //         'result' => $charge
                // ];
            }
            else{
                $redirectData['error_name'] = "Not Human Error";
                $redirectData['error_message']  = "Please reload your page and try again. If the issue persists please contact Quivers at 805-250-4555";
                return [
                    'error_code' => 1,
                    'result' => $redirectData
                ];
            }
            // check the status of payment intent, check if it needs further authentication.
            if ($intent["status"]== "requires_action"|| $intent["status"]== "requires_source_action"){
                if (!$intent->next_action || $intent->next_action->type !== 'redirect_to_url') {
                    $redirectData['error_name'] = "Redirect Error";
                    $redirectData['error_message']  = 'Failed to prepare the payment flow. Please try again. If the issue persists please contact Quivers at 805-250-4555';
                    return ['error_code' => 1,
                            'result' => $redirectData
                        ];

                }
                return [
                    'error_code' => 0,
                    'result' => $intent,
                    '3D_Implement' => 1
                ];
            }
            else if ($intent["status"]== "requires_capture"){
                return [
                    'error_code' => 0,
                    'result' => $intent,
                    '3D_Implement' => 0
                ];
            }

        } 
        catch (\Stripe\Error\ApiConnection $e) {
            // Network problem, perhaps try again.
            $redirectData['error_name'] = "Network problem";
            $redirectData['error_message']  = 'Unable to connect to payment gateway server. Please try again. If the issue persists please contact Quivers at 805-250-4555';
            return ['error_code' => 1,
                    'result' => $redirectData
                ];

        } catch (\Stripe\Error\InvalidRequest $e) {
            // You screwed up in your programming. Shouldn't happen!
            $redirectData['error_name'] = "Invalid Request";
            $redirectData['error_message']  = 'Please try again. If the issue persists please contact Quivers at 805-250-4555';
            return ['error_code' => 1,
                    'result' => $redirectData
                ];

        } catch (\Stripe\Error\Api $e) {
            // Stripe's servers are down!
            $redirectData['error_name'] = "Connection Error";
            $redirectData['error_message']  = 'Unable to connect to payment gateway server. Please try again. If the issue persists please contact Quivers at 805-250-4555';
            return ['error_code' => 1,
                    'result' => $redirectData
                ];

        } catch (\Stripe\Error\Card $e) {
            // Card error.
            $redirectData['error_name'] = "Card error";
            $redirectData['error_message']  = 'Please verify the details before trying again. If the issue persists please contact Quivers at 805-250-4555';
            return ['error_code' => 1,
                    'result' => $redirectData
                ];
        }catch(\Exception $e) {
            //Exception
            $redirectData['error_name'] = "Error";
            $redirectData['error_message']  = $e->getMessage().' Please verify the details before trying again. If the issue persists please contact Quivers at 805-250-4555';
            return ['error_code' => 1,
                    'result' => $redirectData
            ];
        }

    }


    /**
     * Format amount.
    */
    public function formatCurrencyAmount($amount, $currency)
    {
        // Stripe takes amounts as the smallest currency unit. ...
        if (empty($amount)) {
            return null;
        }
        $multiple = 100;
        if (in_array(
            $currency,
            array('BIF', 'DJF', 'JPY', 'KRW', 'PYG', 'VND', 'XAF', 'XPF', 'CLP', 'GNF', 'KMF', 'MGA', 'RWF', 'VUV', 'XOF'),
            true
        )) {
            $multiple = 1;
        }

        return sprintf('%01.0f', (float)$amount * $multiple);
    }

    
    /**
     * Assembles and returns a shopware URL that can be used e.g. for a redirect return.
     *
     * @param array $components
     * @return string
     */
    protected function assembleShopwareUrl(array $components)
    {
        $front = $this->get('front');
        $url = $front->Router()->assemble($components);
        if (!preg_match('#^https?://#', $url)) {
            if (mb_strpos($url, '/') !== 0) {
                $url = $front->Request()->getBaseUrl() . '/' . $url;
            }
            $uri = $front->Request()->getScheme() . '://' . $front->Request()->getHttpHost();
            $url = $uri . $url;
        }
        return $url;
    }
    
    /**
     * @param string $key
     * @return mixed
     */
    protected function get($key)
    {
        return Shopware()->Container()->get($key);
    }


    /**
     * Retrieve Payment intent. Check if it has authentication has been passed
    */
    public function verifyStripeAuthentication($paymentIntentId)
    {
        if (!$paymentIntentId){
            return [];
        }
        $secret_key = $this->settingsService->get('secret_key', 'stripe_settings');
        $paymentIntent  = [];        
        try {
            \Stripe\Stripe::setApiKey($secret_key);
            $paymentIntent = \Stripe\PaymentIntent::retrieve($paymentIntentId);
        }
        catch (Exception $e) {
        }
        if (!$paymentIntent) {
            return [];
        }
        if ($paymentIntent["status"]== "requires_capture"){
            return $paymentIntent;
        }
        else if ($paymentIntent["status"]== "requires_payment_method" || $paymentIntent["status"]== "requires_source") {
            return [];
        }
        else {
            return [];
        }
    }
}