<?php

use Quivers\Components\QuiversStripe;
use Shopware\Models\Order\Order;


class Shopware_Controllers_Frontend_StripePayment extends Shopware_Controllers_Frontend_Payment
{

    public function indexAction()
    { 
        if(!$this->is_available()){
            $redirectData = [];
            $redirectData['error_name'] = "Minimium order total";
            $redirectData['error_message']  ="The Payment Method is not available for use as it is not fully configured yet";
            $this->redirectToShippingPayment($redirectData);
            return;
        }
        switch ($this->getPaymentShortName()) {
            case 'Quivers Stripe':
                return $this->redirect(['action' => 'authorize', 'forceSecure' => true]);
            default:
                return $this->redirect(['controller' => 'checkout']);
        }
    }


    /**
     * Create Stripe Customer, Create Payment Intent, Check if 3D secure Authentication is required.
     * If 3D secure Authentication is not required save the order in db and complete payment process
    */
    public function authorizeAction()
    {
        $service = $this->container->get('quivers_plugin.quivers_stripe');
        //If service commes null
        if (!$service){
            $redirectData = [];
            $redirectData['error_name'] = "Stripe Payment Error";
            $redirectData['error_message']  ="Unable to process payment. Please try again. If the issue persists please contact Quivers at 805-250-4555";
            $this->redirectToShippingPayment($redirectData);
            return;    
        }

        // Get amount and cuurency, send it to QuiversStripe to create payment intent.
        $amount = $this->getAmount();
        $currency = $this->getCurrencyShortName();
        $data = ["amount"=>$amount, "currency"=>$currency];
        $response = $service->createPaymentResponse($this->getUser(), $data);

        //Check if an error has occurred during payment intent creation, redirect to payment page in  this case
        if($response["error_code"] == 1){
            $this->redirectToShippingPayment($response["result"]);
            return;
        }
        //Check if 3D secure Authentication is required
        $intent = $response["result"];
        if($response["error_code"] == 0 && $response["3D_Implement"]){
            $this->redirect($intent->next_action->redirect_to_url->url);

        }
        // Case: 3D secure Authentication is not required, complete payment process
        if($response["error_code"] == 0 && !$response["3D_Implement"]){
            $order = $this->saveOrderWithPaymentIntent($intent);
            $this->finishCheckout();
        }
    }

    /**
     * Save the order with payment intent id and source in db.
    */
     protected function saveOrderWithPaymentIntent(Stripe\PaymentIntent $intent)
    {
        $orderNumber = $this->saveOrder(
            $intent->charges->data[0]->id, 
            $intent->source, 
            0
        );
        if (!$orderNumber) {
            return null;
        }
        // Update the cleared date
        $order = $this->get('models')->getRepository('Shopware\\Models\\Order\\Order')->findOneBy([
            'number' => $orderNumber,
        ]);
        $order->setClearedDate(new \DateTime());
        $this->get('models')->flush($order);
        try {
            // Save the order number in the intent description
            $intent->description .= ' / Order ' . $orderNumber;
            $intent->save();
        } catch (Exception $e) {
        }
        return $order;
    }

    /**
     * This action for creating the return URL of a Stripe redirect flow.
    */
    public function completeRedirectFlowAction()
    {
        // Try to get the Stripe payment intent ID
        $paymentIntentId = $this->Request()->getParam('payment_intent');
        $service = $this->container->get('quivers_plugin.quivers_stripe');
        if (!$service){
            $redirectData = [];
            $redirectData['error_name'] = "Stripe Payment Error";
            $redirectData['error_message']  ="Unable to process payment. Please try again. If the issue persists please contact Quivers at 805-250-4555";
            $this->redirectToShippingPayment($redirectData);
            return;    
        }
        $paymentIntent = $service->verifyStripeAuthentication($paymentIntentId);
        if (!empty($paymentIntent)){
            $order = $this->saveOrderWithPaymentIntent($paymentIntent);
            $this->finishCheckout();
        }
        else{
            $redirectData = [];
            $redirectData['error_name'] = "Stripe Payment Error";
            $redirectData['error_message']  = "The payment processed was cancelled. Please try again. If the issue persists please contact Quivers at 805-250-4555";
            $this->redirectToShippingPayment($redirectData);
            return;    
        }


    }

    /**
     * Redirect to finish action to complete payment process.
    */
    protected function finishCheckout()
    {
        Shopware()->Container()->get('session')->Quiversstripe = new \ArrayObject([], \ArrayObject::STD_PROP_LIST);
        $this->redirect([
            'controller' => 'checkout',
            'action' => 'finish'
        ]);
    }


    /**
     * Check if amount < minimum order total, raise error.
    */
    public function is_available() {
        $basket =  $this->getBasket();
        $settingService = $this->container->get('quivers_plugin.setting_service');
        if (!$settingService){
            return false;    
        }
        $minAmount = $settingService->get('min_order_total', 'stripe_settings');
        $amount = (float)$minAmount;
        if ( $basket['AmountNetNumeric']  < $amount ) {
            return false;
        }
        return true;
    }

    /**
     * Redirect the page to shipping payment page.
    */
    public function redirectToShippingPayment($data){
        $redirectData = [
            'controller' => 'checkout',
            'action' => 'shippingPayment',
            'error_code' => 1,
        ];
        $redirectData['error_name'] = $data['error_name'];
        $redirectData['error_message']  = $data['error_message'];

        $this->redirect($redirectData);

    }
}